@@ -41,18 +41,14 @@ ATOMIC_T ly_log_opts = (uint_fast32_t)(LY_LOLOG | LY_LOSTORE_LAST);
 THREAD_LOCAL uint32_t *temp_ly_log_opts;
 static ly_log_clb log_clb;
 static ATOMIC_T path_flag = 1;
-THREAD_LOCAL char last_msg[LY_LAST_MSG_SIZE];
 #ifndef NDEBUG
 ATOMIC_T ly_ldbg_groups = 0;
 #endif
 
 THREAD_LOCAL struct ly_log_location_s log_location = {0};
 
-LIBYANG_API_DEF const char *
-ly_last_errmsg(void)
-{
-    return last_msg;
-}
+/* how many bytes add when enlarging buffers */
+#define LY_BUF_STEP 128
 
 LIBYANG_API_DEF LY_ERR
 ly_errcode(const struct ly_ctx *ctx)
@@ -173,148 +169,61 @@ ly_err_new(struct ly_err_item **err, LY_ERR ecode, LY_VECODE vecode, char *path,
     return e->no;
 }
 
-/**
- * @brief Get error record from error hash table of a context for the current thread.
- *
- * @param[in] ctx Context to use.
- * @return Thread error record, if any.
- */
-static struct ly_ctx_err_rec *
-ly_err_get_rec(const struct ly_ctx *ctx)
-{
-    struct ly_ctx_err_rec rec, *match;
-
-    /* prepare record */
-    rec.tid = pthread_self();
-
-    /* get the pointer to the matching record */
-    if (lyht_find(ctx->err_ht, &rec, lyht_hash((void *)&rec.tid, sizeof rec.tid), (void **)&match)) {
-        return NULL;
-    }
-
-    return match;
-}
-
-/**
- * @brief Insert new error record to error hash table of a context for the current thread.
- *
- * @param[in] ctx Context to use.
- * @return Thread error record.
- */
-static struct ly_ctx_err_rec *
-ly_err_new_rec(const struct ly_ctx *ctx)
-{
-    struct ly_ctx_err_rec new, *rec;
-    LY_ERR r;
-
-    /* insert a new record */
-    new.err = NULL;
-    new.tid = pthread_self();
-
-    /* reuse lock */
-    /* LOCK */
-    pthread_mutex_lock((pthread_mutex_t *)&ctx->lyb_hash_lock);
-
-    r = lyht_insert(ctx->err_ht, &new, lyht_hash((void *)&new.tid, sizeof new.tid), (void **)&rec);
-
-    /* UNLOCK */
-    pthread_mutex_unlock((pthread_mutex_t *)&ctx->lyb_hash_lock);
-
-    return r ? NULL : rec;
-}
-
 LIBYANG_API_DEF struct ly_err_item *
 ly_err_first(const struct ly_ctx *ctx)
 {
-    struct ly_ctx_err_rec *rec;
-
     LY_CHECK_ARG_RET(NULL, ctx, NULL);
 
-    /* get the pointer to the matching record */
-    rec = ly_err_get_rec(ctx);
-
-    return rec ? rec->err : NULL;
+    return pthread_getspecific(ctx->errlist_key);
 }
 
 LIBYANG_API_DEF struct ly_err_item *
 ly_err_last(const struct ly_ctx *ctx)
 {
-    struct ly_ctx_err_rec *rec;
+    const struct ly_err_item *e;
 
     LY_CHECK_ARG_RET(NULL, ctx, NULL);
 
-    /* get the pointer to the matching record */
-    if (!(rec = ly_err_get_rec(ctx))) {
-        return NULL;
-    }
-
-    return rec->err ? rec->err->prev : NULL;
-}
-
-void
-ly_err_move(struct ly_ctx *src_ctx, struct ly_ctx *trg_ctx)
-{
-    struct ly_ctx_err_rec *rec;
-    struct ly_err_item *err = NULL;
-
-    /* get and remove the errors from src */
-    rec = ly_err_get_rec(src_ctx);
-    if (rec) {
-        err = rec->err;
-        rec->err = NULL;
-    }
-
-    /* set them for trg */
-    if (!(rec = ly_err_get_rec(trg_ctx))) {
-        if (!(rec = ly_err_new_rec(trg_ctx))) {
-            LOGINT(NULL);
-            ly_err_free(err);
-            return;
-        }
-    }
-    ly_err_free(rec->err);
-    rec->err = err;
+    e = pthread_getspecific(ctx->errlist_key);
+    return e ? e->prev : NULL;
 }
 
 LIBYANG_API_DEF void
 ly_err_free(void *ptr)
 {
-    struct ly_err_item *e, *next;
+    struct ly_err_item *i, *next;
 
     /* clean the error list */
-    LY_LIST_FOR_SAFE(ptr, next, e) {
-        free(e->msg);
-        free(e->path);
-        free(e->apptag);
-        free(e);
+    for (i = (struct ly_err_item *)ptr; i; i = next) {
+        next = i->next;
+        free(i->msg);
+        free(i->path);
+        free(i->apptag);
+        free(i);
     }
 }
 
 LIBYANG_API_DEF void
 ly_err_clean(struct ly_ctx *ctx, struct ly_err_item *eitem)
 {
-    struct ly_ctx_err_rec *rec;
-    struct ly_err_item *e;
+    struct ly_err_item *i, *first;
 
-    if (!(rec = ly_err_get_rec(ctx))) {
-        return;
-    }
-    if (rec->err == eitem) {
+    first = ly_err_first(ctx);
+    if (first == eitem) {
         eitem = NULL;
     }
-
-    if (!eitem) {
-        /* free all err */
-        ly_err_free(rec->err);
-        rec->err = NULL;
-    } else {
+    if (eitem) {
         /* disconnect the error */
-        for (e = rec->err; e && (e->next != eitem); e = e->next) {}
-        assert(e);
-        e->next = NULL;
-        rec->err->prev = e;
+        for (i = first; i && (i->next != eitem); i = i->next) {}
+        assert(i);
+        i->next = NULL;
+        first->prev = i;
         /* free this err and newer */
         ly_err_free(eitem);
+    } else {
+        /* free all err */
+        ly_err_free(first);
+        pthread_setspecific(ctx->errlist_key, NULL);
     }
 }
 
@@ -427,83 +336,64 @@ ly_log_location_revert(uint32_t scnode_steps, uint32_t dnode_steps, uint32_t pat
     }
 }
 
-/**
- * @brief Store generated error in a context.
- *
- * @param[in] ctx Context to use.
- * @param[in] level Message log level.
- * @param[in] no Error number.
- * @param[in] vecode Error validation error code.
- * @param[in] msg Error message, always spent.
- * @param[in] path Error path, always spent.
- * @param[in] apptag Error app tag, always spent.
- * @return LY_ERR value.
- */
 static LY_ERR
 log_store(const struct ly_ctx *ctx, LY_LOG_LEVEL level, LY_ERR no, LY_VECODE vecode, char *msg, char *path, char *apptag)
 {
-    struct ly_ctx_err_rec *rec;
-    struct ly_err_item *e, *last;
+    struct ly_err_item *eitem, *last;
 
     assert(ctx && (level < LY_LLVRB));
 
-    if (!(rec = ly_err_get_rec(ctx))) {
-        if (!(rec = ly_err_new_rec(ctx))) {
-            goto mem_fail;
-        }
-    }
-
-    e = rec->err;
-    if (!e) {
+    eitem = pthread_getspecific(ctx->errlist_key);
+    if (!eitem) {
         /* if we are only to fill in path, there must have been an error stored */
         assert(msg);
-        e = malloc(sizeof *e);
-        LY_CHECK_GOTO(!e, mem_fail);
-        e->prev = e;
-        e->next = NULL;
+        eitem = malloc(sizeof *eitem);
+        LY_CHECK_GOTO(!eitem, mem_fail);
+        eitem->prev = eitem;
+        eitem->next = NULL;
 
-        rec->err = e;
+        pthread_setspecific(ctx->errlist_key, eitem);
     } else if (!msg) {
         /* only filling the path */
         assert(path);
 
         /* find last error */
-        e = e->prev;
+        eitem = eitem->prev;
         do {
-            if (e->level == LY_LLERR) {
+            if (eitem->level == LY_LLERR) {
                 /* fill the path */
-                free(e->path);
-                e->path = path;
+                free(eitem->path);
+                eitem->path = path;
                 return LY_SUCCESS;
             }
-            e = e->prev;
-        } while (e->prev->next);
+            eitem = eitem->prev;
+        } while (eitem->prev->next);
         /* last error was not found */
         assert(0);
     } else if ((temp_ly_log_opts && ((*temp_ly_log_opts & LY_LOSTORE_LAST) == LY_LOSTORE_LAST)) ||
             (!temp_ly_log_opts && ((ATOMIC_LOAD_RELAXED(ly_log_opts) & LY_LOSTORE_LAST) == LY_LOSTORE_LAST))) {
         /* overwrite last message */
-        free(e->msg);
-        free(e->path);
-        free(e->apptag);
+        free(eitem->msg);
+        free(eitem->path);
+        free(eitem->apptag);
     } else {
         /* store new message */
-        last = e->prev;
-        e->prev = malloc(sizeof *e);
-        LY_CHECK_GOTO(!e->prev, mem_fail);
-        e = e->prev;
-        e->prev = last;
-        e->next = NULL;
-        last->next = e;
+        last = eitem->prev;
+        eitem->prev = malloc(sizeof *eitem);
+        LY_CHECK_GOTO(!eitem->prev, mem_fail);
+        eitem = eitem->prev;
+        eitem->prev = last;
+        eitem->next = NULL;
+        last->next = eitem;
     }
 
     /* fill in the information */
-    e->level = level;
-    e->no = no;
-    e->vecode = vecode;
-    e->msg = msg;
-    e->path = path;
-    e->apptag = apptag;
+    eitem->level = level;
+    eitem->no = no;
+    eitem->vecode = vecode;
+    eitem->msg = msg;
+    eitem->path = path;
+    eitem->apptag = apptag;
     return LY_SUCCESS;
 
 mem_fail:
@@ -555,19 +445,14 @@ log_vprintf(const struct ly_ctx *ctx, LY_LOG_LEVEL level, LY_ERR no, LY_VECODE v
         return;
     }
 
-    /* print into a single message */
-    if (vasprintf(&msg, format, args) == -1) {
-        LOGMEM(ctx);
-        free(path);
-        return;
-    }
-
-    /* store as the last message */
-    strncpy(last_msg, msg, LY_LAST_MSG_SIZE - 1);
-
-    /* store the error/warning in the context (if we need to store errors internally, it does not matter what are
-     * the user log options) */
+    /* store the error/warning (if we need to store errors internally, it does not matter what are the user log options) */
     if ((level < LY_LLVRB) && ctx && lostore) {
+        assert(format);
+        if (vasprintf(&msg, format, args) == -1) {
+            LOGMEM(ctx);
+            free(path);
+            return;
+        }
         if (((no & ~LY_EPLUGIN) == LY_EVALID) && (vecode == LYVE_SUCCESS)) {
             /* assume we are inheriting the error, so inherit vecode as well */
             vecode = ly_vecode(ctx);
@@ -577,6 +462,11 @@ log_vprintf(const struct ly_ctx *ctx, LY_LOG_LEVEL level, LY_ERR no, LY_VECODE v
         }
         free_strs = 0;
     } else {
+        if (vasprintf(&msg, format, args) == -1) {
+            LOGMEM(ctx);
+            free(path);
+            return;
+        }
         free_strs = 1;
     }
 
@@ -708,8 +598,8 @@ ly_vlog_build_path(const struct ly_ctx *ctx, char **path)
 
     if (log_location.paths.count && ((const char *)(log_location.paths.objs[log_location.paths.count - 1]))[0]) {
         /* simply get what is in the provided path string */
-        r = asprintf(path, "Path \"%s\".", (const char *)log_location.paths.objs[log_location.paths.count - 1]);
-        LY_CHECK_ERR_RET(r == -1, LOGMEM(ctx), LY_EMEM);
+        *path = strdup((const char *)log_location.paths.objs[log_location.paths.count - 1]);
+        LY_CHECK_ERR_RET(!(*path), LOGMEM(ctx), LY_EMEM);
     } else {
         /* data/schema node */
         if (log_location.dnodes.count) {
@@ -790,12 +680,12 @@ ly_vlog(const struct ly_ctx *ctx, const char *apptag, LY_VECODE code, const char
  * @param[in] plugin_name Name of the plugin generating the message.
  * @param[in] level Log message level (error, warning, etc.)
  * @param[in] err_no Error type code.
- * @param[in] path Optional path of the error, used if set.
+ * @param[in] path Optional path of the error.
  * @param[in] format Format string to print.
  * @param[in] ap Var arg list for @p format.
  */
 static void
-ly_ext_log(const struct ly_ctx *ctx, const char *plugin_name, LY_LOG_LEVEL level, LY_ERR err_no, char *path,
+ly_ext_log(const struct ly_ctx *ctx, const char *plugin_name, LY_LOG_LEVEL level, LY_ERR err_no, const char *path,
         const char *format, va_list ap)
 {
     char *plugin_msg;
@@ -808,7 +698,8 @@ ly_ext_log(const struct ly_ctx *ctx, const char *plugin_name, LY_LOG_LEVEL level
         return;
     }
 
-    log_vprintf(ctx, level, (level == LY_LLERR ? LY_EPLUGIN : 0) | err_no, LYVE_OTHER, path, NULL, plugin_msg, ap);
+    log_vprintf(ctx, level, (level == LY_LLERR ? LY_EPLUGIN : 0) | err_no, LYVE_OTHER, path ? strdup(path) : NULL, NULL,
+            plugin_msg, ap);
     free(plugin_msg);
 }
 
@@ -826,6 +717,8 @@ lyplg_ext_parse_log(const struct lysp_ctx *pctx, const struct lysp_ext_instance
     va_start(ap, format);
     ly_ext_log(PARSER_CTX(pctx), ext->record->plugin.id, level, err_no, path, format, ap);
     va_end(ap);
+
+    free(path);
 }
 
 LIBYANG_API_DEF void
@@ -833,15 +726,9 @@ lyplg_ext_compile_log(const struct lysc_ctx *cctx, const struct lysc_ext_instanc
         const char *format, ...)
 {
     va_list ap;
-    char *path = NULL;
-
-    if (cctx && (asprintf(&path, "Path \"%s\".", cctx->path) == -1)) {
-        LOGMEM(cctx->ctx);
-        return;
-    }
 
     va_start(ap, format);
-    ly_ext_log(ext->module->ctx, ext->def->plugin->id, level, err_no, path, format, ap);
+    ly_ext_log(ext->module->ctx, ext->def->plugin->id, level, err_no, cctx ? cctx->path : NULL, format, ap);
     va_end(ap);
 }
 
@@ -850,37 +737,12 @@ lyplg_ext_compile_log_path(const char *path, const struct lysc_ext_instance *ext
         const char *format, ...)
 {
     va_list ap;
-    char *log_path = NULL;
-
-    if (path && (asprintf(&log_path, "Path \"%s\".", path) == -1)) {
-        LOGMEM(ext->module->ctx);
-        return;
-    }
 
     va_start(ap, format);
-    ly_ext_log(ext->module->ctx, ext->def->plugin->id, level, err_no, log_path, format, ap);
+    ly_ext_log(ext->module->ctx, ext->def->plugin->id, level, err_no, path, format, ap);
     va_end(ap);
 }
 
-/**
- * @brief Serves only for creating ap.
- */
-static void
-_lyplg_ext_compile_log_err(const struct ly_err_item *err, const struct lysc_ext_instance *ext, ...)
-{
-    va_list ap;
-
-    va_start(ap, ext);
-    ly_ext_log(ext->module->ctx, ext->def->plugin->id, err->level, err->no, err->path ? strdup(err->path) : NULL, "%s", ap);
-    va_end(ap);
-}
-
-LIBYANG_API_DEF void
-lyplg_ext_compile_log_err(const struct ly_err_item *err, const struct lysc_ext_instance *ext)
-{
-    _lyplg_ext_compile_log_err(err, ext, err->msg);
-}
-
 /**
  * @brief Exact same functionality as ::ly_err_print() but has variable arguments so log_vprintf() can be called.
  */
