@@ -39,8 +39,8 @@
 #define REALLOC_CHUNK(NEW_SIZE) \
     NEW_SIZE + (1024 - (NEW_SIZE % 1024))
 
-ly_bool
-ly_should_print(const struct lyd_node *node, uint32_t options)
+LIBYANG_API_DEF ly_bool
+lyd_node_should_print(const struct lyd_node *node, uint32_t options)
 {
     const struct lyd_node *elem;
 
@@ -63,7 +63,7 @@ ly_should_print(const struct lyd_node *node, uint32_t options)
 
         /* avoid empty default containers */
         LYD_TREE_DFS_BEGIN(node, elem) {
-            if ((elem != node) && ly_should_print(elem, options)) {
+            if ((elem != node) && lyd_node_should_print(elem, options)) {
                 return 1;
             }
             assert(elem->flags & LYD_DEFAULT);
@@ -89,14 +89,14 @@ ly_should_print(const struct lyd_node *node, uint32_t options)
     return 1;
 }
 
-API LY_OUT_TYPE
+LIBYANG_API_DEF LY_OUT_TYPE
 ly_out_type(const struct ly_out *out)
 {
     LY_CHECK_ARG_RET(NULL, out, LY_OUT_ERROR);
     return out->type;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_out_new_clb(ly_write_clb writeclb, void *user_data, struct ly_out **out)
 {
     LY_CHECK_ARG_RET(NULL, out, writeclb, LY_EINVAL);
@@ -111,7 +111,7 @@ ly_out_new_clb(ly_write_clb writeclb, void *user_data, struct ly_out **out)
     return LY_SUCCESS;
 }
 
-API ly_write_clb
+LIBYANG_API_DEF ly_write_clb
 ly_out_clb(struct ly_out *out, ly_write_clb writeclb)
 {
     ly_write_clb prev_clb;
@@ -127,7 +127,7 @@ ly_out_clb(struct ly_out *out, ly_write_clb writeclb)
     return prev_clb;
 }
 
-API void *
+LIBYANG_API_DEF void *
 ly_out_clb_arg(struct ly_out *out, void *arg)
 {
     void *prev_arg;
@@ -143,7 +143,7 @@ ly_out_clb_arg(struct ly_out *out, void *arg)
     return prev_arg;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_out_new_fd(int fd, struct ly_out **out)
 {
     LY_CHECK_ARG_RET(NULL, out, fd != -1, LY_EINVAL);
@@ -156,7 +156,7 @@ ly_out_new_fd(int fd, struct ly_out **out)
     return LY_SUCCESS;
 }
 
-API int
+LIBYANG_API_DEF int
 ly_out_fd(struct ly_out *out, int fd)
 {
     int prev_fd;
@@ -198,7 +198,7 @@ ly_out_fd(struct ly_out *out, int fd)
     return prev_fd;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_out_new_file(FILE *f, struct ly_out **out)
 {
     LY_CHECK_ARG_RET(NULL, out, f, LY_EINVAL);
@@ -212,7 +212,7 @@ ly_out_new_file(FILE *f, struct ly_out **out)
     return LY_SUCCESS;
 }
 
-API FILE *
+LIBYANG_API_DEF FILE *
 ly_out_file(struct ly_out *out, FILE *f)
 {
     FILE *prev_f;
@@ -228,7 +228,7 @@ ly_out_file(struct ly_out *out, FILE *f)
     return prev_f;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_out_new_memory(char **strp, size_t size, struct ly_out **out)
 {
     LY_CHECK_ARG_RET(NULL, out, strp, LY_EINVAL);
@@ -274,7 +274,7 @@ ly_out_memory(struct ly_out *out, char **strp, size_t size)
     return data;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_out_reset(struct ly_out *out)
 {
     LY_CHECK_ARG_RET(NULL, out, LY_EINVAL);
@@ -320,7 +320,7 @@ ly_out_reset(struct ly_out *out)
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_out_new_filepath(const char *filepath, struct ly_out **out)
 {
     LY_CHECK_ARG_RET(NULL, out, filepath, LY_EINVAL);
@@ -329,7 +329,7 @@ ly_out_new_filepath(const char *filepath, struct ly_out **out)
     LY_CHECK_ERR_RET(!*out, LOGMEM(NULL), LY_EMEM);
 
     (*out)->type = LY_OUT_FILEPATH;
-    (*out)->method.fpath.f = fopen(filepath, "w");
+    (*out)->method.fpath.f = fopen(filepath, "wb");
     if (!(*out)->method.fpath.f) {
         LOGERR(NULL, LY_ESYS, "Failed to open file \"%s\" (%s).", filepath, strerror(errno));
         free(*out);
@@ -340,7 +340,7 @@ ly_out_new_filepath(const char *filepath, struct ly_out **out)
     return LY_SUCCESS;
 }
 
-API const char *
+LIBYANG_API_DEF const char *
 ly_out_filepath(struct ly_out *out, const char *filepath)
 {
     FILE *f;
@@ -353,7 +353,7 @@ ly_out_filepath(struct ly_out *out, const char *filepath)
 
     /* replace filepath */
     f = out->method.fpath.f;
-    out->method.fpath.f = fopen(filepath, "w");
+    out->method.fpath.f = fopen(filepath, "wb");
     if (!out->method.fpath.f) {
         LOGERR(NULL, LY_ESYS, "Failed to open file \"%s\" (%s).", filepath, strerror(errno));
         out->method.fpath.f = f;
@@ -366,7 +366,7 @@ ly_out_filepath(struct ly_out *out, const char *filepath)
     return NULL;
 }
 
-API void
+LIBYANG_API_DEF void
 ly_out_free(struct ly_out *out, void (*clb_arg_destructor)(void *arg), ly_bool destroy)
 {
     if (!out) {
@@ -493,7 +493,7 @@ ly_print_(struct ly_out *out, const char *format, ...)
     return ret;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_print(struct ly_out *out, const char *format, ...)
 {
     LY_ERR ret;
@@ -508,7 +508,7 @@ ly_print(struct ly_out *out, const char *format, ...)
     return ret;
 }
 
-API void
+LIBYANG_API_DEF void
 ly_print_flush(struct ly_out *out)
 {
     switch (out->type) {
@@ -590,6 +590,7 @@ repeat:
         break;
     case LY_OUT_FD: {
         ssize_t r;
+
         r = write(out->method.fd, buf, len);
         if (r < 0) {
             ret = LY_ESYS;
@@ -608,6 +609,7 @@ repeat:
         break;
     case LY_OUT_CALLBACK: {
         ssize_t r;
+
         r = out->method.clb.func(out->method.clb.arg, buf, len);
         if (r < 0) {
             ret = LY_ESYS;
@@ -645,7 +647,7 @@ repeat:
     return ret;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 ly_write(struct ly_out *out, const char *buf, size_t len)
 {
     out->func_printed = 0;
@@ -653,7 +655,7 @@ ly_write(struct ly_out *out, const char *buf, size_t len)
     return ly_write_(out, buf, len);
 }
 
-API size_t
+LIBYANG_API_DEF size_t
 ly_out_printed(const struct ly_out *out)
 {
     return out->func_printed;
