@@ -1,9 +1,10 @@
 /**
  * @file plugins_types.c
  * @author Radek Krejci <rkrejci@cesnet.cz>
+ * @author Michal Vasko <mvasko@cesnet.cz>
  * @brief Built-in types plugins and interface for user types plugins.
  *
- * Copyright (c) 2019 CESNET, z.s.p.o.
+ * Copyright (c) 2019 - 2022 CESNET, z.s.p.o.
  *
  * This source code is licensed under BSD 3-Clause License (the "License").
  * You may not use this file except in compliance with the License.
@@ -16,6 +17,7 @@
 
 #include "plugins_types.h"
 
+#include <assert.h>
 #include <ctype.h>
 #include <inttypes.h>
 #include <stdio.h>
@@ -78,7 +80,7 @@ ly_schema_resolved_resolve_prefix(const struct ly_ctx *UNUSED(ctx), const char *
     LY_ARRAY_COUNT_TYPE u;
 
     LY_ARRAY_FOR(prefixes, u) {
-        if (!ly_strncmp(prefixes[u].prefix, prefix, prefix_len)) {
+        if ((!prefixes[u].prefix && !prefix_len) || (prefixes[u].prefix && !ly_strncmp(prefixes[u].prefix, prefix, prefix_len))) {
             return prefixes[u].mod;
         }
     }
@@ -134,6 +136,7 @@ ly_resolve_prefix(const struct ly_ctx *ctx, const void *prefix, size_t prefix_le
         mod = ly_schema_resolved_resolve_prefix(ctx, prefix, prefix_len, prefix_data);
         break;
     case LY_VALUE_XML:
+    case LY_VALUE_STR_NS:
         mod = ly_xml_resolve_prefix(ctx, prefix, prefix_len, prefix_data);
         break;
     case LY_VALUE_CANON:
@@ -146,23 +149,26 @@ ly_resolve_prefix(const struct ly_ctx *ctx, const void *prefix, size_t prefix_le
     return mod;
 }
 
-API const struct lys_module *
-lyplg_type_identity_module(const struct ly_ctx *ctx, const struct lysc_node *ctx_node,
-        const char *prefix, size_t prefix_len, LY_VALUE_FORMAT format, const void *prefix_data)
+LIBYANG_API_DEF const struct lys_module *
+lyplg_type_identity_module(const struct ly_ctx *ctx, const struct lysc_node *ctx_node, const char *prefix,
+        size_t prefix_len, LY_VALUE_FORMAT format, const void *prefix_data)
 {
     if (prefix_len) {
         return ly_resolve_prefix(ctx, prefix, prefix_len, format, prefix_data);
     } else {
         switch (format) {
         case LY_VALUE_SCHEMA:
+            /* use local module */
+            return ly_schema_resolve_prefix(ctx, prefix, prefix_len, prefix_data);
         case LY_VALUE_SCHEMA_RESOLVED:
-            /* use context node module, handles augments */
-            return ctx_node->module;
+            /* use local module */
+            return ly_schema_resolved_resolve_prefix(ctx, prefix, prefix_len, prefix_data);
         case LY_VALUE_CANON:
         case LY_VALUE_JSON:
         case LY_VALUE_LYB:
+        case LY_VALUE_STR_NS:
             /* use context node module (as specified) */
-            return ctx_node->module;
+            return ctx_node ? ctx_node->module : NULL;
         case LY_VALUE_XML:
             /* use the default namespace */
             return ly_xml_resolve_prefix(ctx, NULL, 0, prefix_data);
@@ -251,6 +257,7 @@ ly_get_prefix(const struct lys_module *mod, LY_VALUE_FORMAT format, void *prefix
         prefix = ly_schema_resolved_get_prefix(mod, prefix_data);
         break;
     case LY_VALUE_XML:
+    case LY_VALUE_STR_NS:
         prefix = ly_xml_get_prefix(mod, prefix_data);
         break;
     case LY_VALUE_CANON:
@@ -263,13 +270,13 @@ ly_get_prefix(const struct lys_module *mod, LY_VALUE_FORMAT format, void *prefix
     return prefix;
 }
 
-API const char *
+LIBYANG_API_DEF const char *
 lyplg_type_get_prefix(const struct lys_module *mod, LY_VALUE_FORMAT format, void *prefix_data)
 {
     return ly_get_prefix(mod, format, prefix_data);
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_compare_simple(const struct lyd_value *val1, const struct lyd_value *val2)
 {
     if (val1->realtype != val2->realtype) {
@@ -283,7 +290,7 @@ lyplg_type_compare_simple(const struct lyd_value *val1, const struct lyd_value *
     return LY_ENOT;
 }
 
-API const void *
+LIBYANG_API_DEF const void *
 lyplg_type_print_simple(const struct ly_ctx *UNUSED(ctx), const struct lyd_value *value, LY_VALUE_FORMAT UNUSED(format),
         void *UNUSED(prefix_data), ly_bool *dynamic, size_t *value_len)
 {
@@ -296,7 +303,7 @@ lyplg_type_print_simple(const struct ly_ctx *UNUSED(ctx), const struct lyd_value
     return value->_canonical;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_dup_simple(const struct ly_ctx *ctx, const struct lyd_value *original, struct lyd_value *dup)
 {
     memset(dup, 0, sizeof *dup);
@@ -306,14 +313,14 @@ lyplg_type_dup_simple(const struct ly_ctx *ctx, const struct lyd_value *original
     return LY_SUCCESS;
 }
 
-API void
+LIBYANG_API_DEF void
 lyplg_type_free_simple(const struct ly_ctx *ctx, struct lyd_value *value)
 {
     lydict_remove(ctx, value->_canonical);
     value->_canonical = NULL;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_parse_int(const char *datatype, int base, int64_t min, int64_t max, const char *value, size_t value_len,
         int64_t *ret, struct ly_err_item **err)
 {
@@ -325,20 +332,22 @@ lyplg_type_parse_int(const char *datatype, int base, int64_t min, int64_t max, c
     for ( ; value_len && isspace(*value); ++value, --value_len) {}
 
     if (!value || !value_len || !value[0]) {
-        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid empty %s value.", datatype);
+        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid type %s empty value.", datatype);
     }
 
     switch (ly_parse_int(value, value_len, min, max, base, ret)) {
     case LY_EDENIED:
-        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Value is out of %s's min/max bounds.", datatype);
+        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL,
+                "Value \"%.*s\" is out of type %s min/max bounds.", (int)value_len, value, datatype);
     case LY_SUCCESS:
         return LY_SUCCESS;
     default:
-        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid %s value \"%.*s\".", datatype, (int)value_len, value);
+        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL,
+                "Invalid type %s value \"%.*s\".", datatype, (int)value_len, value);
     }
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_parse_uint(const char *datatype, int base, uint64_t max, const char *value, size_t value_len, uint64_t *ret,
         struct ly_err_item **err)
 {
@@ -350,23 +359,23 @@ lyplg_type_parse_uint(const char *datatype, int base, uint64_t max, const char *
     for ( ; value_len && isspace(*value); ++value, --value_len) {}
 
     if (!value || !value_len || !value[0]) {
-        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid empty %s value.", datatype);
+        return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid type %s empty value.", datatype);
     }
 
     *err = NULL;
     switch (ly_parse_uint(value, value_len, max, base, ret)) {
     case LY_EDENIED:
         return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL,
-                "Value \"%.*s\" is out of %s's min/max bounds.", (int)value_len, value, datatype);
+                "Value \"%.*s\" is out of type %s min/max bounds.", (int)value_len, value, datatype);
     case LY_SUCCESS:
         return LY_SUCCESS;
     default:
         return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL,
-                "Invalid %s value \"%.*s\".", datatype, (int)value_len, value);
+                "Invalid type %s value \"%.*s\".", datatype, (int)value_len, value);
     }
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_parse_dec64(uint8_t fraction_digits, const char *value, size_t value_len, int64_t *ret, struct ly_err_item **err)
 {
     LY_ERR ret_val;
@@ -426,6 +435,7 @@ decimal:
     if (len + trailing_zeros < value_len) {
         /* consume trailing whitespaces to check that there is nothing after it */
         uint64_t u;
+
         for (u = len + trailing_zeros; u < value_len && isspace(value[u]); ++u) {}
         if (u != value_len) {
             return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL,
@@ -461,7 +471,7 @@ decimal:
     return ret_val;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_validate_patterns(struct lysc_pattern **patterns, const char *str, size_t str_len, struct ly_err_item **err)
 {
     int rc, match_opts;
@@ -489,6 +499,7 @@ lyplg_type_validate_patterns(struct lysc_pattern **patterns, const char *str, si
 
         if ((rc != PCRE2_ERROR_NOMATCH) && (rc < 0)) {
             PCRE2_UCHAR pcre2_errmsg[LY_PCRE2_MSG_LIMIT] = {0};
+
             pcre2_get_error_message(rc, pcre2_errmsg, LY_PCRE2_MSG_LIMIT);
 
             return ly_err_new(err, LY_ESYS, 0, NULL, NULL, "%s", (const char *)pcre2_errmsg);
@@ -500,6 +511,7 @@ lyplg_type_validate_patterns(struct lysc_pattern **patterns, const char *str, si
                 return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, eapptag, "%s", patterns[u]->emsg);
             } else {
                 const char *inverted = patterns[u]->inverted ? "inverted " : "";
+
                 return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, eapptag,
                         LY_ERRMSG_NOPATTERN, (int)str_len, str, inverted, patterns[u]->expr);
             }
@@ -508,7 +520,7 @@ lyplg_type_validate_patterns(struct lysc_pattern **patterns, const char *str, si
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_validate_range(LY_DATA_TYPE basetype, struct lysc_range *range, int64_t value, const char *strval,
         size_t strval_len, struct ly_err_item **err)
 {
@@ -523,6 +535,7 @@ lyplg_type_validate_range(LY_DATA_TYPE basetype, struct lysc_range *range, int64
             /* unsigned */
             if ((uint64_t)value < range->parts[u].min_u64) {
                 char *eapptag = range->eapptag ? strdup(range->eapptag) : NULL;
+
                 if (range->emsg) {
                     return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, eapptag, "%s", range->emsg);
                 } else {
@@ -535,6 +548,7 @@ lyplg_type_validate_range(LY_DATA_TYPE basetype, struct lysc_range *range, int64
             } else if (u == LY_ARRAY_COUNT(range->parts) - 1) {
                 /* we have the last range part, so the value is out of bounds */
                 char *eapptag = range->eapptag ? strdup(range->eapptag) : NULL;
+
                 if (range->emsg) {
                     return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, eapptag, "%s", range->emsg);
                 } else {
@@ -546,6 +560,7 @@ lyplg_type_validate_range(LY_DATA_TYPE basetype, struct lysc_range *range, int64
             /* signed */
             if (value < range->parts[u].min_64) {
                 char *eapptag = range->eapptag ? strdup(range->eapptag) : NULL;
+
                 if (range->emsg) {
                     return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, eapptag, "%s", range->emsg);
                 } else {
@@ -557,6 +572,7 @@ lyplg_type_validate_range(LY_DATA_TYPE basetype, struct lysc_range *range, int64
             } else if (u == LY_ARRAY_COUNT(range->parts) - 1) {
                 /* we have the last range part, so the value is out of bounds */
                 char *eapptag = range->eapptag ? strdup(range->eapptag) : NULL;
+
                 if (range->emsg) {
                     return ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, eapptag, "%s", range->emsg);
                 } else {
@@ -569,7 +585,7 @@ lyplg_type_validate_range(LY_DATA_TYPE basetype, struct lysc_range *range, int64
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_prefix_data_new(const struct ly_ctx *ctx, const void *value, size_t value_len, LY_VALUE_FORMAT format,
         const void *prefix_data, LY_VALUE_FORMAT *format_p, void **prefix_data_p)
 {
@@ -579,7 +595,7 @@ lyplg_type_prefix_data_new(const struct ly_ctx *ctx, const void *value, size_t v
     return ly_store_prefix_data(ctx, value, value_len, format, prefix_data, format_p, (void **)prefix_data_p);
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_prefix_data_dup(const struct ly_ctx *ctx, LY_VALUE_FORMAT format, const void *orig, void **dup)
 {
     LY_CHECK_ARG_RET(NULL, dup, LY_EINVAL);
@@ -592,7 +608,7 @@ lyplg_type_prefix_data_dup(const struct ly_ctx *ctx, LY_VALUE_FORMAT format, con
     return ly_dup_prefix_data(ctx, format, orig, (void **)dup);
 }
 
-API void
+LIBYANG_API_DEF void
 lyplg_type_prefix_data_free(LY_VALUE_FORMAT format, void *prefix_data)
 {
     ly_free_prefix_data(format, prefix_data);
@@ -615,7 +631,7 @@ type_get_hints_base(uint32_t hints)
     }
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_check_hints(uint32_t hints, const char *value, size_t value_len, LY_DATA_TYPE type, int *base,
         struct ly_err_item **err)
 {
@@ -684,7 +700,7 @@ lyplg_type_check_hints(uint32_t hints, const char *value, size_t value_len, LY_D
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_check_status(const struct lysc_node *ctx_node, uint16_t val_flags, LY_VALUE_FORMAT format, void *prefix_data,
         const char *val_name, struct ly_err_item **err)
 {
@@ -719,7 +735,7 @@ lyplg_type_check_status(const struct lysc_node *ctx_node, uint16_t val_flags, LY
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_lypath_check_status(const struct lysc_node *ctx_node, const struct ly_path *path, LY_VALUE_FORMAT format,
         void *prefix_data, struct ly_err_item **err)
 {
@@ -759,7 +775,7 @@ lyplg_type_lypath_check_status(const struct lysc_node *ctx_node, const struct ly
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_lypath_new(const struct ly_ctx *ctx, const char *value, size_t value_len, uint32_t options,
         LY_VALUE_FORMAT format, void *prefix_data, const struct lysc_node *ctx_node, struct lys_glob_unres *unres,
         struct ly_path **path, struct ly_err_item **err)
@@ -782,6 +798,7 @@ lyplg_type_lypath_new(const struct ly_ctx *ctx, const char *value, size_t value_
     case LY_VALUE_CANON:
     case LY_VALUE_LYB:
     case LY_VALUE_JSON:
+    case LY_VALUE_STR_NS:
         prefix_opt = LY_PATH_PREFIX_STRICT_INHERIT;
         break;
     }
@@ -818,13 +835,13 @@ cleanup:
     return ret;
 }
 
-API void
+LIBYANG_API_DEF void
 lyplg_type_lypath_free(const struct ly_ctx *ctx, struct ly_path *path)
 {
     ly_path_free(ctx, path);
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_make_implemented(struct lys_module *mod, const char **features, struct lys_glob_unres *unres)
 {
     if (mod->implemented) {
@@ -837,11 +854,13 @@ lyplg_type_make_implemented(struct lys_module *mod, const char **features, struc
     return LY_SUCCESS;
 }
 
-API LY_ERR
+LIBYANG_API_DEF LY_ERR
 lyplg_type_identity_isderived(const struct lysc_ident *base, const struct lysc_ident *der)
 {
     LY_ARRAY_COUNT_TYPE u;
 
+    assert(base->module->ctx == der->module->ctx);
+
     LY_ARRAY_FOR(base->derived, u) {
         if (der == base->derived[u]) {
             return LY_SUCCESS;
@@ -853,64 +872,172 @@ lyplg_type_identity_isderived(const struct lysc_ident *base, const struct lysc_i
     return LY_ENOTFOUND;
 }
 
-API LY_ERR
+/**
+ * @brief Try to generate a path to the leafref target with its value to enable the use of hash-based search.
+ *
+ * @param[in] path Leafref path.
+ * @param[in] ctx_node Leafref context node.
+ * @param[in] format Format of @p path.
+ * @param[in] prefix_data Prefix data of @p path.
+ * @param[in] target_val Leafref target value.
+ * @param[out] target_path Generated path with the target value.
+ * @return LY_SUCCESS on success.
+ * @return LY_ENOT if no matching target exists.
+ * @return LY_ERR on error.
+ */
+static LY_ERR
+lyplg_type_resolve_leafref_get_target_path(const struct lyxp_expr *path, const struct lysc_node *ctx_node,
+        LY_VALUE_FORMAT format, void *prefix_data, const char *target_val, struct lyxp_expr **target_path)
+{
+    LY_ERR rc = LY_SUCCESS;
+    uint8_t oper;
+    struct ly_path *p = NULL;
+    char *str_path = NULL, quot;
+    int len;
+    ly_bool list_key = 0;
+
+    *target_path = NULL;
+
+    /* compile, has already been so it must succeed */
+    oper = (ctx_node->flags & LYS_IS_OUTPUT) ? LY_PATH_OPER_OUTPUT : LY_PATH_OPER_INPUT;
+    if (ly_path_compile_leafref(ctx_node->module->ctx, ctx_node, NULL, path, oper, LY_PATH_TARGET_MANY, format,
+            prefix_data, &p)) {
+        /* the target was found before but is disabled so it was removed */
+        return LY_ENOT;
+    }
+
+    /* check whether we can search for a list instance with a specific key value */
+    if (lysc_is_key(p[LY_ARRAY_COUNT(p) - 1].node)) {
+        if ((LY_ARRAY_COUNT(p) >= 2) && (p[LY_ARRAY_COUNT(p) - 2].node->nodetype == LYS_LIST)) {
+            if ((path->tokens[path->used - 1] == LYXP_TOKEN_NAMETEST) &&
+                    (path->tokens[path->used - 2] == LYXP_TOKEN_OPER_PATH) &&
+                    (path->tokens[path->used - 3] == LYXP_TOKEN_NAMETEST)) {
+                list_key = 1;
+            } /* else again, should be possible but does not make sense */
+        } /* else allowed despite not making sense */
+    }
+
+    if (list_key) {
+        /* get the length of the orig expression without the last "/" and the key node */
+        len = path->tok_pos[path->used - 3] + path->tok_len[path->used - 3];
+
+        /* generate the string path evaluated using hashes */
+        quot = strchr(target_val, '\'') ? '\"' : '\'';
+        if (asprintf(&str_path, "%.*s[%s=%c%s%c]/%s", len, path->expr, path->expr + path->tok_pos[path->used - 1],
+                quot, target_val, quot, path->expr + path->tok_pos[path->used - 1]) == -1) {
+            LOGMEM(ctx_node->module->ctx);
+            rc = LY_EMEM;
+            goto cleanup;
+        }
+
+    } else {
+        /* leaf will not be found using hashes, but generate the path just to unify it */
+        assert(p[LY_ARRAY_COUNT(p) - 1].node->nodetype & LYD_NODE_TERM);
+
+        /* generate the string path evaluated using hashes */
+        quot = strchr(target_val, '\'') ? '\"' : '\'';
+        if (asprintf(&str_path, "%s[.=%c%s%c]", path->expr, quot, target_val, quot) == -1) {
+            LOGMEM(ctx_node->module->ctx);
+            rc = LY_EMEM;
+            goto cleanup;
+        }
+    }
+
+    /* parse into an expression */
+    LY_CHECK_GOTO(lyxp_expr_parse(ctx_node->module->ctx, str_path, 0, 1, target_path), cleanup);
+
+cleanup:
+    ly_path_free(ctx_node->module->ctx, p);
+    free(str_path);
+    return rc;
+}
+
+LIBYANG_API_DEF LY_ERR
 lyplg_type_resolve_leafref(const struct lysc_type_leafref *lref, const struct lyd_node *node, struct lyd_value *value,
         const struct lyd_node *tree, struct lyd_node **target, char **errmsg)
 {
-    LY_ERR ret;
+    LY_ERR rc = LY_SUCCESS;
+    struct lyxp_expr *target_path = NULL;
     struct lyxp_set set = {0};
-    const char *val_str;
+    const char *val_str, *xp_err_msg;
     uint32_t i;
-    int rc;
+    int r;
 
     LY_CHECK_ARG_RET(NULL, lref, node, value, errmsg, LY_EINVAL);
 
-    /* find all target data instances */
-    ret = lyxp_eval(LYD_CTX(node), lref->path, node->schema->module, LY_VALUE_SCHEMA_RESOLVED, lref->prefixes,
-            node, tree, NULL, &set, LYXP_IGNORE_WHEN);
-    if (ret) {
-        ret = LY_ENOTFOUND;
-        val_str = lref->plugin->print(LYD_CTX(node), value, LY_VALUE_CANON, NULL, NULL, NULL);
-        if (asprintf(errmsg, "Invalid leafref value \"%s\" - XPath evaluation error.", val_str) == -1) {
-            *errmsg = NULL;
-            ret = LY_EMEM;
-        }
-        goto error;
+    if (target) {
+        *target = NULL;
     }
 
-    /* check whether any matches */
-    for (i = 0; i < set.used; ++i) {
-        if (set.val.nodes[i].type != LYXP_NODE_ELEM) {
-            continue;
-        }
+    /* get the canonical value */
+    val_str = lyd_value_get_canonical(LYD_CTX(node), value);
 
-        if (!lref->plugin->compare(&((struct lyd_node_term *)set.val.nodes[i].node)->value, value)) {
-            break;
+    if (!strchr(val_str, '\"') || !strchr(val_str, '\'')) {
+        /* get the path with the value */
+        r = lyplg_type_resolve_leafref_get_target_path(lref->path, node->schema, LY_VALUE_SCHEMA_RESOLVED, lref->prefixes,
+                val_str, &target_path);
+        if (r == LY_ENOT) {
+            goto cleanup;
+        } else if (r) {
+            rc = r;
+            goto cleanup;
         }
-    }
-    if (i == set.used) {
-        ret = LY_ENOTFOUND;
-        val_str = lref->plugin->print(LYD_CTX(node), value, LY_VALUE_CANON, NULL, NULL, NULL);
-        if (set.used) {
-            rc = asprintf(errmsg, LY_ERRMSG_NOLREF_VAL, val_str, lref->path->expr);
+    } /* else value with both ' and ", XPath does not support that */
+
+    /* find the target data instance(s) */
+    rc = lyxp_eval(LYD_CTX(node), target_path ? target_path : lref->path, node->schema->module,
+            LY_VALUE_SCHEMA_RESOLVED, lref->prefixes, node, node, tree, NULL, &set, LYXP_IGNORE_WHEN);
+    if (rc) {
+        if (ly_errcode(LYD_CTX(node)) == rc) {
+            xp_err_msg = ly_errmsg(LYD_CTX(node));
         } else {
-            rc = asprintf(errmsg, LY_ERRMSG_NOLREF_INST, val_str, lref->path->expr);
+            xp_err_msg = NULL;
         }
-        if (rc == -1) {
+
+        if (xp_err_msg) {
+            r = asprintf(errmsg, "Invalid leafref value \"%s\" - XPath evaluation error (%s).", val_str, xp_err_msg);
+        } else {
+            r = asprintf(errmsg, "Invalid leafref value \"%s\" - XPath evaluation error.", val_str);
+        }
+        if (r == -1) {
             *errmsg = NULL;
-            ret = LY_EMEM;
+            rc = LY_EMEM;
         }
-        goto error;
+        goto cleanup;
     }
 
+    /* check the result */
+    if (target_path) {
+        /* no or exact match(es) */
+        i = 0;
+    } else {
+        /* check whether any matches */
+        for (i = 0; i < set.used; ++i) {
+            if (set.val.nodes[i].type != LYXP_NODE_ELEM) {
+                continue;
+            }
+
+            if (!lref->plugin->compare(&((struct lyd_node_term *)set.val.nodes[i].node)->value, value)) {
+                break;
+            }
+        }
+    }
+
+    if (i == set.used) {
+        /* no match found */
+        rc = LY_ENOTFOUND;
+        if (asprintf(errmsg, LY_ERRMSG_NOLREF_VAL, val_str, lref->path->expr) == -1) {
+            *errmsg = NULL;
+            rc = LY_EMEM;
+        }
+        goto cleanup;
+    }
     if (target) {
         *target = set.val.nodes[i].node;
     }
 
+cleanup:
+    lyxp_expr_free(LYD_CTX(node), target_path);
     lyxp_set_free_content(&set);
-    return LY_SUCCESS;
-
-error:
-    lyxp_set_free_content(&set);
-    return ret;
+    return rc;
 }
