@@ -172,8 +172,6 @@ union_store_type(const struct ly_ctx *ctx, struct lysc_type *type, struct lyd_va
     const void *value = NULL;
     size_t value_len = 0;
 
-    *err = NULL;
-
     if (subvalue->format == LY_VALUE_LYB) {
         lyb_parse_union(subvalue->original, subvalue->orig_len, NULL, &value, &value_len);
     } else {
@@ -222,63 +220,36 @@ union_find_type(const struct ly_ctx *ctx, struct lysc_type **types, struct lyd_v
 {
     LY_ERR ret = LY_SUCCESS;
     LY_ARRAY_COUNT_TYPE u;
-    struct ly_err_item **errs = NULL, *e;
     uint32_t temp_lo = 0;
-    char *msg = NULL;
-    int msg_len = 0;
-
-    *err = NULL;
 
     if (!types || !LY_ARRAY_COUNT(types)) {
         return LY_EINVAL;
     }
 
-    /* alloc errors */
-    errs = calloc(LY_ARRAY_COUNT(types), sizeof *errs);
-    LY_CHECK_RET(!errs, LY_EMEM);
+    *err = NULL;
 
     /* turn logging temporarily off */
     ly_temp_log_options(&temp_lo);
 
     /* use the first usable subtype to store the value */
     for (u = 0; u < LY_ARRAY_COUNT(types); ++u) {
-        ret = union_store_type(ctx, types[u], subvalue, resolve, ctx_node, tree, unres, &e);
+        ret = union_store_type(ctx, types[u], subvalue, resolve, ctx_node, tree, unres, err);
         if ((ret == LY_SUCCESS) || (ret == LY_EINCOMPLETE)) {
             break;
         }
 
-        errs[u] = e;
+        ly_err_free(*err);
+        *err = NULL;
     }
 
     if (u == LY_ARRAY_COUNT(types)) {
-        /* create the full error */
-        msg_len = asprintf(&msg, "Invalid union value \"%.*s\" - no matching subtype found:\n",
+        ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "Invalid union value \"%.*s\" - no matching subtype found.",
                 (int)subvalue->orig_len, (char *)subvalue->original);
-        if (msg_len == -1) {
-            LY_CHECK_ERR_GOTO(!errs, ret = LY_EMEM, cleanup);
-        }
-        for (u = 0; u < LY_ARRAY_COUNT(types); ++u) {
-            if (!errs[u]) {
-                /* no error for some reason */
-                continue;
-            }
-
-            msg = ly_realloc(msg, msg_len + 4 + strlen(types[u]->plugin->id) + 2 + strlen(errs[u]->msg) + 2);
-            LY_CHECK_ERR_GOTO(!msg, ret = LY_EMEM, cleanup);
-            msg_len += sprintf(msg + msg_len, "    %s: %s\n", types[u]->plugin->id, errs[u]->msg);
-        }
-
-        ret = ly_err_new(err, LY_EVALID, LYVE_DATA, NULL, NULL, "%s", msg);
     } else if (type_idx) {
         *type_idx = u;
     }
 
-cleanup:
-    for (u = 0; u < LY_ARRAY_COUNT(types); ++u) {
-        ly_err_free(errs[u]);
-    }
-    free(errs);
-    free(msg);
+    /* restore logging */
     ly_temp_log_options(NULL);
     return ret;
 }
@@ -310,26 +281,29 @@ lyb_fill_subvalue(const struct ly_ctx *ctx, struct lysc_type_union *type_u, cons
     ret = lyb_union_validate(lyb_data, lyb_data_len, type_u, err);
     LY_CHECK_RET(ret);
 
-    /* parse lyb_data and set the lyb_value and lyb_value_len */
+    /* Parse lyb_data and set the lyb_value and lyb_value_len. */
     lyb_parse_union(lyb_data, lyb_data_len, &type_idx, &lyb_value, &lyb_value_len);
     LY_CHECK_RET(ret);
 
-    /* store lyb_data to subvalue */
-    ret = union_subvalue_assignment(lyb_data, lyb_data_len, &subvalue->original, &subvalue->orig_len, options);
+    /* Store lyb_data to subvalue. */
+    ret = union_subvalue_assignment(lyb_data, lyb_data_len,
+            &subvalue->original, &subvalue->orig_len, options);
     LY_CHECK_RET(ret);
 
     if (lyb_value) {
-        /* resolve prefix_data and set format */
+        /* Resolve prefix_data and set format. */
         ret = lyplg_type_prefix_data_new(ctx, lyb_value, lyb_value_len, LY_VALUE_LYB, prefix_data, &subvalue->format,
                 &subvalue->prefix_data);
         LY_CHECK_RET(ret);
         assert(subvalue->format == LY_VALUE_LYB);
     } else {
-        /* lyb_parse_union() did not find lyb_value, just set format */
+        /* The lyb_parse_union() did not find lyb_value.
+         * Just set format.
+         */
         subvalue->format = LY_VALUE_LYB;
     }
 
-    /* use the specific type to store the value */
+    /* Use the specific type to store the value. */
     ret = union_store_type(ctx, type_u->types[type_idx], subvalue, 0, NULL, NULL, unres, err);
 
     return ret;
@@ -355,11 +329,13 @@ lyplg_type_store_union(const struct ly_ctx *ctx, const struct lysc_type *type, c
     subvalue->ctx_node = ctx_node;
 
     if (format == LY_VALUE_LYB) {
-        ret = lyb_fill_subvalue(ctx, type_u, value, value_len, prefix_data, subvalue, &options, unres, err);
+        ret = lyb_fill_subvalue(ctx, type_u, value, value_len,
+                prefix_data, subvalue, &options, unres, err);
         LY_CHECK_GOTO((ret != LY_SUCCESS) && (ret != LY_EINCOMPLETE), cleanup);
     } else {
         /* Store @p value to subvalue. */
-        ret = union_subvalue_assignment(value, value_len, &subvalue->original, &subvalue->orig_len, &options);
+        ret = union_subvalue_assignment(value, value_len,
+                &subvalue->original, &subvalue->orig_len, &options);
         LY_CHECK_GOTO(ret, cleanup);
 
         /* store format-specific data for later prefix resolution */
@@ -394,7 +370,6 @@ lyplg_type_validate_union(const struct ly_ctx *ctx, const struct lysc_type *type
     LY_ERR ret = LY_SUCCESS;
     struct lysc_type_union *type_u = (struct lysc_type_union *)type;
     struct lyd_value_union *subvalue = storage->subvalue;
-    uint32_t type_idx;
 
     *err = NULL;
 
@@ -403,16 +378,9 @@ lyplg_type_validate_union(const struct ly_ctx *ctx, const struct lysc_type *type
      * we have to perform union value storing again from scratch */
     subvalue->value.realtype->plugin->free(ctx, &subvalue->value);
 
-    if (subvalue->format == LY_VALUE_LYB) {
-        /* use the specific type to store the value */
-        lyb_parse_union(subvalue->original, 0, &type_idx, NULL, NULL);
-        ret = union_store_type(ctx, type_u->types[type_idx], subvalue, 1, ctx_node, tree, NULL, err);
-        LY_CHECK_RET(ret);
-    } else {
-        /* use the first usable subtype to store the value */
-        ret = union_find_type(ctx, type_u->types, subvalue, 1, ctx_node, tree, NULL, NULL, err);
-        LY_CHECK_RET(ret);
-    }
+    /* use the first usable subtype to store the value */
+    ret = union_find_type(ctx, type_u->types, subvalue, 1, ctx_node, tree, NULL, NULL, err);
+    LY_CHECK_RET(ret);
 
     /* success, update the canonical value, if any generated */
     lydict_remove(ctx, storage->_canonical);
@@ -423,6 +391,10 @@ lyplg_type_validate_union(const struct ly_ctx *ctx, const struct lysc_type *type
 LIBYANG_API_DEF LY_ERR
 lyplg_type_compare_union(const struct lyd_value *val1, const struct lyd_value *val2)
 {
+    if (val1->realtype != val2->realtype) {
+        return LY_ENOT;
+    }
+
     if (val1->subvalue->value.realtype != val2->subvalue->value.realtype) {
         return LY_ENOT;
     }
@@ -446,10 +418,10 @@ lyb_union_print(const struct ly_ctx *ctx, struct lysc_type_union *type_u, struct
         void *prefix_data, size_t *value_len)
 {
     void *ret = NULL;
-    LY_ERR r;
+    LY_ERR retval;
     struct ly_err_item *err;
     uint64_t num = 0;
-    uint32_t type_idx = 0;
+    uint32_t type_idx;
     ly_bool dynamic;
     size_t pval_len;
     void *pval;
@@ -463,9 +435,8 @@ lyb_union_print(const struct ly_ctx *ctx, struct lysc_type_union *type_u, struct
         ctx = subvalue->ctx_node->module->ctx;
     }
     subvalue->value.realtype->plugin->free(ctx, &subvalue->value);
-    r = union_find_type(ctx, type_u->types, subvalue, 0, NULL, NULL, &type_idx, NULL, &err);
-    ly_err_free(err);
-    LY_CHECK_RET((r != LY_SUCCESS) && (r != LY_EINCOMPLETE), NULL);
+    retval = union_find_type(ctx, type_u->types, subvalue, 0, NULL, NULL, &type_idx, NULL, &err);
+    LY_CHECK_RET((retval != LY_SUCCESS) && (retval != LY_EINCOMPLETE), NULL);
 
     /* Print subvalue in LYB format. */
     pval = (void *)subvalue->value.realtype->plugin->print(NULL, &subvalue->value, LY_VALUE_LYB, prefix_data, &dynamic,
