@@ -134,14 +134,10 @@ lyd_create_inner(const struct lysc_node *schema, struct lyd_node **node)
 }
 
 LY_ERR
-lyd_create_list(const struct lysc_node *schema, const struct ly_path_predicate *predicates, const struct lyxp_var *vars,
-        struct lyd_node **node)
+lyd_create_list(const struct lysc_node *schema, const struct ly_path_predicate *predicates, struct lyd_node **node)
 {
     LY_ERR ret = LY_SUCCESS;
     struct lyd_node *list = NULL, *key;
-    const struct lyd_value *value;
-    struct lyd_value val = {0};
-    struct lyxp_var *var;
     LY_ARRAY_COUNT_TYPE u;
 
     assert((schema->nodetype == LYS_LIST) && !(schema->flags & LYS_KEYLESS));
@@ -153,31 +149,7 @@ lyd_create_list(const struct lysc_node *schema, const struct ly_path_predicate *
 
     /* create and insert all the keys */
     LY_ARRAY_FOR(predicates, u) {
-        if (predicates[u].type == LY_PATH_PREDTYPE_LIST_VAR) {
-            /* find the var */
-            if ((ret = lyxp_vars_find(schema->module->ctx, vars, predicates[u].variable, 0, &var))) {
-                goto cleanup;
-            }
-
-            /* store the value */
-            LOG_LOCSET(predicates[u].key, NULL, NULL, NULL);
-            ret = lyd_value_store(schema->module->ctx, &val, ((struct lysc_node_leaf *)predicates[u].key)->type,
-                    var->value, strlen(var->value), NULL, LY_VALUE_JSON, NULL, LYD_HINT_DATA, predicates[u].key, NULL);
-            LOG_LOCBACK(1, 0, 0, 0);
-            LY_CHECK_GOTO(ret, cleanup);
-
-            value = &val;
-        } else {
-            assert(predicates[u].type == LY_PATH_PREDTYPE_LIST);
-            value = &predicates[u].value;
-        }
-
-        ret = lyd_create_term2(predicates[u].key, value, &key);
-        if (val.realtype) {
-            val.realtype->plugin->free(schema->module->ctx, &val);
-            memset(&val, 0, sizeof val);
-        }
-        LY_CHECK_GOTO(ret, cleanup);
+        LY_CHECK_GOTO(ret = lyd_create_term2(predicates[u].key, &predicates[u].value, &key), cleanup);
         lyd_insert_node(list, NULL, key, 0);
     }
 
@@ -200,6 +172,7 @@ lyd_create_list2(const struct lysc_node *schema, const char *keys, size_t keys_l
     LY_ERR ret = LY_SUCCESS;
     struct lyxp_expr *expr = NULL;
     uint32_t exp_idx = 0;
+    enum ly_path_pred_type pred_type = 0;
     struct ly_path_predicate *predicates = NULL;
 
     LOG_LOCSET(schema, NULL, NULL, NULL);
@@ -210,111 +183,100 @@ lyd_create_list2(const struct lysc_node *schema, const char *keys, size_t keys_l
 
     /* compile them */
     LY_CHECK_GOTO(ret = ly_path_compile_predicate(schema->module->ctx, NULL, NULL, schema, expr, &exp_idx, LY_VALUE_JSON,
-            NULL, &predicates), cleanup);
+            NULL, &predicates, &pred_type), cleanup);
 
     /* create the list node */
-    LY_CHECK_GOTO(ret = lyd_create_list(schema, predicates, NULL, node), cleanup);
+    LY_CHECK_GOTO(ret = lyd_create_list(schema, predicates, node), cleanup);
 
 cleanup:
     LOG_LOCBACK(1, 0, 0, 0);
     lyxp_expr_free(schema->module->ctx, expr);
-    ly_path_predicates_free(schema->module->ctx, predicates);
+    ly_path_predicates_free(schema->module->ctx, pred_type, predicates);
     return ret;
 }
 
 /**
- * @brief Learn actual any value type in case it is currently ::LYD_ANYDATA_STRING.
- *
- * @param[in] value Any value.
- * @param[out] value_type Detected value type.
- */
-static void
-lyd_create_any_string_valtype(const void *value, LYD_ANYDATA_VALUETYPE *value_type)
-{
-    /* detect format */
-    if (!value) {
-        /* interpret it as an empty data tree */
-        *value_type = LYD_ANYDATA_DATATREE;
-    } else if (((char *)value)[0] == '<') {
-        *value_type = LYD_ANYDATA_XML;
-    } else if (((char *)value)[0] == '{') {
-        *value_type = LYD_ANYDATA_JSON;
-    } else if (!strncmp(value, "lyb", 3)) {
-        *value_type = LYD_ANYDATA_LYB;
-    } else {
-        /* really just some string */
-        *value_type = LYD_ANYDATA_STRING;
-    }
-}
-
-/**
- * @brief Convert an any value into a datatree.
+ * @brief Convert an anydata value into a datatree.
  *
  * @param[in] ctx libyang context.
- * @param[in] value_in Any value as an input.
- * @param[in] value_type Any @p value type.
- * @param[in] log Whether parsing errors should be logged.
+ * @param[in] value Anydata value.
+ * @param[in] value_type Anydata @p value type.
  * @param[out] tree Parsed data tree.
  * @return LY_ERR value.
  */
 static LY_ERR
-lyd_create_any_datatree(const struct ly_ctx *ctx, struct ly_in *value_in, LYD_ANYDATA_VALUETYPE value_type, ly_bool log,
+lyd_create_anydata_datatree(const struct ly_ctx *ctx, const void *value, LYD_ANYDATA_VALUETYPE value_type,
         struct lyd_node **tree)
 {
-    LY_ERR rc = LY_SUCCESS;
+    LY_ERR r;
+    struct ly_in *in = NULL;
     struct lyd_ctx *lydctx = NULL;
-    uint32_t parse_opts, int_opts, log_opts = 0;
+    uint32_t parse_opts, int_opts;
 
     *tree = NULL;
 
+    if (!value) {
+        /* empty data tree no matter the value type */
+        return LY_SUCCESS;
+    }
+
+    if (value_type == LYD_ANYDATA_STRING) {
+        /* detect format */
+        if (((char *)value)[0] == '<') {
+            value_type = LYD_ANYDATA_XML;
+        } else if (((char *)value)[0] == '{') {
+            value_type = LYD_ANYDATA_JSON;
+        } else if (!strncmp(value, "lyb", 3)) {
+            value_type = LYD_ANYDATA_LYB;
+        } else {
+            LOGERR(ctx, LY_EINVAL, "Invalid string value of an anydata node.");
+            return LY_EINVAL;
+        }
+    }
+
+    /* create input */
+    LY_CHECK_RET(ly_in_new_memory(value, &in));
+
     /* set options */
     parse_opts = LYD_PARSE_ONLY | LYD_PARSE_OPAQ;
     int_opts = LYD_INTOPT_ANY | LYD_INTOPT_WITH_SIBLINGS;
 
-    if (!log) {
-        /* no logging */
-        ly_temp_log_options(&log_opts);
-    }
-
     switch (value_type) {
     case LYD_ANYDATA_DATATREE:
     case LYD_ANYDATA_STRING:
         /* unreachable */
+        ly_in_free(in, 0);
         LOGINT_RET(ctx);
     case LYD_ANYDATA_XML:
-        rc = lyd_parse_xml(ctx, NULL, NULL, tree, value_in, parse_opts, 0, int_opts, NULL, NULL, &lydctx);
+        r = lyd_parse_xml(ctx, NULL, NULL, tree, in, parse_opts, 0, int_opts, NULL, NULL, &lydctx);
         break;
     case LYD_ANYDATA_JSON:
-        rc = lyd_parse_json(ctx, NULL, NULL, tree, value_in, parse_opts, 0, int_opts, NULL, NULL, &lydctx);
+        r = lyd_parse_json(ctx, NULL, NULL, tree, in, parse_opts, 0, int_opts, NULL, NULL, &lydctx);
         break;
     case LYD_ANYDATA_LYB:
-        rc = lyd_parse_lyb(ctx, NULL, NULL, tree, value_in, parse_opts | LYD_PARSE_STRICT, 0, int_opts, NULL, NULL, &lydctx);
+        r = lyd_parse_lyb(ctx, NULL, NULL, tree, in, parse_opts | LYD_PARSE_STRICT, 0, int_opts, NULL, NULL, &lydctx);
         break;
     }
     if (lydctx) {
         lydctx->free(lydctx);
     }
+    ly_in_free(in, 0);
 
-    if (!log) {
-        /* restore logging */
-        ly_temp_log_options(NULL);
+    if (r) {
+        LOGERR(ctx, LY_EINVAL, "Failed to parse anydata content into a data tree.");
+        return LY_EINVAL;
     }
-    if (rc && *tree) {
-        lyd_free_siblings(*tree);
-        *tree = NULL;
-    }
-    return rc;
+    return LY_SUCCESS;
 }
 
 LY_ERR
 lyd_create_any(const struct lysc_node *schema, const void *value, LYD_ANYDATA_VALUETYPE value_type, ly_bool use_value,
         struct lyd_node **node)
 {
-    LY_ERR rc = LY_SUCCESS, r;
+    LY_ERR ret;
     struct lyd_node *tree;
     struct lyd_node_any *any = NULL;
     union lyd_any_value any_val;
-    struct ly_in *in = NULL;
 
     assert(schema->nodetype & LYD_NODE_ANY);
 
@@ -325,70 +287,15 @@ lyd_create_any(const struct lysc_node *schema, const void *value, LYD_ANYDATA_VA
     any->prev = &any->node;
     any->flags = LYD_NEW;
 
-    if (schema->nodetype == LYS_ANYDATA) {
-        /* anydata */
-        if (value_type == LYD_ANYDATA_STRING) {
-            /* detect value type */
-            lyd_create_any_string_valtype(value, &value_type);
-        }
-
-        if (value_type != LYD_ANYDATA_DATATREE) {
-            /* create input */
-            assert(value);
-            LY_CHECK_GOTO(rc = ly_in_new_memory(value, &in), cleanup);
-
-            /* parse as a data tree */
-            if ((r = lyd_create_any_datatree(schema->module->ctx, in, value_type, 1, &tree))) {
-                LOGERR(schema->module->ctx, rc, "Failed to parse any content into a data tree.");
-                rc = r;
-                goto cleanup;
-            }
-
-            /* use the parsed data tree */
-            if (use_value) {
-                free((void *)value);
-            }
-            use_value = 1;
-            value = tree;
-            value_type = LYD_ANYDATA_DATATREE;
-        }
-    } else {
-        /* anyxml */
-        switch (value_type) {
-        case LYD_ANYDATA_DATATREE:
-            /* fine, just use the value */
-            break;
-        case LYD_ANYDATA_STRING:
-            /* detect value type */
-            lyd_create_any_string_valtype(value, &value_type);
-            if ((value_type == LYD_ANYDATA_DATATREE) || (value_type == LYD_ANYDATA_STRING)) {
-                break;
-            }
-        /* fallthrough */
-        case LYD_ANYDATA_XML:
-        case LYD_ANYDATA_JSON:
-        case LYD_ANYDATA_LYB:
-            if (!value) {
-                /* nothing to parse */
-                break;
-            }
-
-            /* create input */
-            LY_CHECK_GOTO(rc = ly_in_new_memory(value, &in), cleanup);
-
-            /* try to parse as a data tree */
-            r = lyd_create_any_datatree(schema->module->ctx, in, value_type, 0, &tree);
-            if (!r) {
-                /* use the parsed data tree */
-                if (use_value) {
-                    free((void *)value);
-                }
-                use_value = 1;
-                value = tree;
-                value_type = LYD_ANYDATA_DATATREE;
-            }
-            break;
+    if ((schema->nodetype == LYS_ANYDATA) && (value_type != LYD_ANYDATA_DATATREE)) {
+        /* only a data tree can be stored */
+        LY_CHECK_GOTO(ret = lyd_create_anydata_datatree(schema->module->ctx, value, value_type, &tree), error);
+        if (use_value) {
+            free((void *)value);
         }
+        use_value = 1;
+        value = tree;
+        value_type = LYD_ANYDATA_DATATREE;
     }
 
     if (use_value) {
@@ -399,7 +306,7 @@ lyd_create_any(const struct lysc_node *schema, const void *value, LYD_ANYDATA_VA
         case LYD_ANYDATA_STRING:
         case LYD_ANYDATA_XML:
         case LYD_ANYDATA_JSON:
-            LY_CHECK_GOTO(rc = lydict_insert_zc(schema->module->ctx, (void *)value, &any->value.str), cleanup);
+            LY_CHECK_GOTO(ret = lydict_insert_zc(schema->module->ctx, (void *)value, &any->value.str), error);
             break;
         case LYD_ANYDATA_LYB:
             any->value.mem = (void *)value;
@@ -408,18 +315,16 @@ lyd_create_any(const struct lysc_node *schema, const void *value, LYD_ANYDATA_VA
         any->value_type = value_type;
     } else {
         any_val.str = value;
-        LY_CHECK_GOTO(rc = lyd_any_copy_value(&any->node, &any_val, value_type), cleanup);
+        LY_CHECK_GOTO(ret = lyd_any_copy_value(&any->node, &any_val, value_type), error);
     }
     lyd_hash(&any->node);
 
-cleanup:
-    if (rc) {
-        lyd_free_tree(&any->node);
-    } else {
-        *node = &any->node;
-    }
-    ly_in_free(in, 0);
-    return rc;
+    *node = &any->node;
+    return LY_SUCCESS;
+
+error:
+    free(any);
+    return ret;
 }
 
 LY_ERR
@@ -1415,20 +1320,18 @@ lyd_new_path_check_find_lypath(struct ly_path *path, const char *str_path, const
         schema = path[u].node;
 
         if (lysc_is_dup_inst_list(schema)) {
-            if (!path[u].predicates ||
-                    ((schema->nodetype == LYS_LEAFLIST) && (path[u].predicates[0].type == LY_PATH_PREDTYPE_LEAFLIST))) {
+            if (path[u].pred_type == LY_PATH_PREDTYPE_NONE) {
                 /* creating a new key-less list or state leaf-list instance */
                 create = 1;
                 new_count = u;
-            } else if (path[u].predicates[0].type != LY_PATH_PREDTYPE_POSITION) {
+            } else if (path[u].pred_type != LY_PATH_PREDTYPE_POSITION) {
                 LOG_LOCSET(schema, NULL, NULL, NULL);
-                LOGVAL(schema->module->ctx, LYVE_XPATH, "Invalid predicate for state %s \"%s\" in path \"%s\".",
+                LOGVAL(schema->module->ctx, LYVE_XPATH, "Invalid predicate for %s \"%s\" in path \"%s\".",
                         lys_nodetype2str(schema->nodetype), schema->name, str_path);
                 LOG_LOCBACK(1, 0, 0, 0);
                 return LY_EINVAL;
             }
-        } else if ((schema->nodetype == LYS_LIST) &&
-                (!path[u].predicates || (path[u].predicates[0].type != LY_PATH_PREDTYPE_LIST))) {
+        } else if ((schema->nodetype == LYS_LIST) && (path[u].pred_type != LY_PATH_PREDTYPE_LIST)) {
             if ((u < LY_ARRAY_COUNT(path) - 1) || !(options & LYD_NEW_PATH_OPAQ)) {
                 LOG_LOCSET(schema, NULL, NULL, NULL);
                 LOGVAL(schema->module->ctx, LYVE_XPATH, "Predicate missing for %s \"%s\" in path \"%s\".",
@@ -1436,8 +1339,7 @@ lyd_new_path_check_find_lypath(struct ly_path *path, const char *str_path, const
                 LOG_LOCBACK(1, 0, 0, 0);
                 return LY_EINVAL;
             } /* else creating an opaque list */
-        } else if ((schema->nodetype == LYS_LEAFLIST) &&
-                (!path[u].predicates || (path[u].predicates[0].type != LY_PATH_PREDTYPE_LEAFLIST))) {
+        } else if ((schema->nodetype == LYS_LEAFLIST) && (path[u].pred_type != LY_PATH_PREDTYPE_LEAFLIST)) {
             r = LY_SUCCESS;
             if (options & LYD_NEW_PATH_OPAQ) {
                 r = lyd_value_validate(NULL, schema, value, value_len, NULL, NULL, NULL);
@@ -1449,8 +1351,8 @@ lyd_new_path_check_find_lypath(struct ly_path *path, const char *str_path, const
                 ++((struct lysc_type *)val.realtype)->refcount;
 
                 /* store the new predicate so that it is used when searching for this instance */
+                path[u].pred_type = LY_PATH_PREDTYPE_LEAFLIST;
                 LY_ARRAY_NEW_RET(schema->module->ctx, path[u].predicates, pred, LY_EMEM);
-                pred->type = LY_PATH_PREDTYPE_LEAFLIST;
                 pred->value = val;
             } /* else we have opaq flag and the value is not valid, leave no predicate and then create an opaque node */
         }
@@ -1540,13 +1442,13 @@ lyd_new_path_(struct lyd_node *parent, const struct ly_ctx *ctx, const struct ly
 
     /* try to find any existing nodes in the path */
     if (parent) {
-        r = ly_path_eval_partial(p, parent, NULL, &path_idx, &node);
-        if (r == LY_SUCCESS) {
+        ret = ly_path_eval_partial(p, parent, &path_idx, &node);
+        if (ret == LY_SUCCESS) {
             if (orig_count == LY_ARRAY_COUNT(p)) {
                 /* the node exists, are we supposed to update it or is it just a default? */
                 if (!(options & LYD_NEW_PATH_UPDATE) && !(node->flags & LYD_DEFAULT)) {
                     LOG_LOCSET(NULL, node, NULL, NULL);
-                    LOGVAL(ctx, LYVE_REFERENCE, "Path \"%s\" already exists.", path);
+                    LOGVAL(ctx, LYVE_REFERENCE, "Path \"%s\" already exists", path);
                     LOG_LOCBACK(0, 1, 0, 0);
                     ret = LY_EEXIST;
                     goto cleanup;
@@ -1556,17 +1458,16 @@ lyd_new_path_(struct lyd_node *parent, const struct ly_ctx *ctx, const struct ly
                 ret = lyd_new_path_update(node, value, value_len, value_type, format, &nparent, &nnode);
                 goto cleanup;
             } /* else we were not searching for the whole path */
-        } else if (r == LY_EINCOMPLETE) {
+        } else if (ret == LY_EINCOMPLETE) {
             /* some nodes were found, adjust the iterator to the next segment */
             ++path_idx;
-        } else if (r == LY_ENOTFOUND) {
+        } else if (ret == LY_ENOTFOUND) {
             /* we will create the nodes from top-level, default behavior (absolute path), or from the parent (relative path) */
             if (lysc_data_parent(p[0].node)) {
                 node = parent;
             }
         } else {
             /* error */
-            ret = r;
             goto cleanup;
         }
     }
@@ -1586,14 +1487,15 @@ lyd_new_path_(struct lyd_node *parent, const struct ly_ctx *ctx, const struct ly
             if (lysc_is_dup_inst_list(schema)) {
                 /* create key-less list instance */
                 LY_CHECK_GOTO(ret = lyd_create_inner(schema, &node), cleanup);
-            } else if ((options & LYD_NEW_PATH_OPAQ) && !p[path_idx].predicates) {
+            } else if ((options & LYD_NEW_PATH_OPAQ) && (p[path_idx].pred_type == LY_PATH_PREDTYPE_NONE)) {
                 /* creating opaque list without keys */
                 LY_CHECK_GOTO(ret = lyd_create_opaq(ctx, schema->name, strlen(schema->name), NULL, 0,
                         schema->module->name, strlen(schema->module->name), NULL, 0, NULL, LY_VALUE_JSON, NULL,
                         LYD_NODEHINT_LIST, &node), cleanup);
             } else {
                 /* create standard list instance */
-                LY_CHECK_GOTO(ret = lyd_create_list(schema, p[path_idx].predicates, NULL, &node), cleanup);
+                assert(p[path_idx].pred_type == LY_PATH_PREDTYPE_LIST);
+                LY_CHECK_GOTO(ret = lyd_create_list(schema, p[path_idx].predicates, &node), cleanup);
             }
             break;
         case LYS_CONTAINER:
@@ -1603,8 +1505,7 @@ lyd_new_path_(struct lyd_node *parent, const struct ly_ctx *ctx, const struct ly
             LY_CHECK_GOTO(ret = lyd_create_inner(schema, &node), cleanup);
             break;
         case LYS_LEAFLIST:
-            if ((options & LYD_NEW_PATH_OPAQ) &&
-                    (!p[path_idx].predicates || (p[path_idx].predicates[0].type != LY_PATH_PREDTYPE_LEAFLIST))) {
+            if ((options & LYD_NEW_PATH_OPAQ) && (p[path_idx].pred_type != LY_PATH_PREDTYPE_LEAFLIST)) {
                 /* we have not checked this only for dup-inst lists, otherwise it must be opaque */
                 r = LY_EVALID;
                 if (lysc_is_dup_inst_list(schema)) {
@@ -1621,13 +1522,13 @@ lyd_new_path_(struct lyd_node *parent, const struct ly_ctx *ctx, const struct ly
             }
 
             /* get value to set */
-            if (p[path_idx].predicates && (p[path_idx].predicates[0].type == LY_PATH_PREDTYPE_LEAFLIST)) {
+            if (p[path_idx].pred_type == LY_PATH_PREDTYPE_LEAFLIST) {
                 val = &p[path_idx].predicates[0].value;
             }
 
             /* create a leaf-list instance */
             if (val) {
-                LY_CHECK_GOTO(ret = lyd_create_term2(schema, val, &node), cleanup);
+                LY_CHECK_GOTO(ret = lyd_create_term2(schema, &p[path_idx].predicates[0].value, &node), cleanup);
             } else {
                 LY_CHECK_GOTO(ret = lyd_create_term(schema, value, value_len, NULL, format, NULL, LYD_HINT_DATA,
                         NULL, &node), cleanup);
