@@ -350,18 +350,19 @@ lysp_type_find(const char *id, struct lysp_node *start_node, const struct lysp_m
  * @param[in,out] ctx Context to log the error.
  * @param[in,out] ht Hash table with top-level names.
  * @param[in] name Inserted top-level identifier.
- * @param[in] statement The name of the statement type from which @p name originated (eg typedef, feature, ...).
+ * @param[in] statement The name of the statement type from which
+ * @p name originated (eg typedef, feature, ...).
  * @param[in] err_detail Optional error specification.
  * @return LY_ERR, but LY_EEXIST is mapped to LY_EVALID.
  */
 static LY_ERR
-lysp_check_dup_ht_insert(struct lysp_ctx *ctx, struct ly_ht *ht, const char *name, const char *statement,
-        const char *err_detail)
+lysp_check_dup_ht_insert(struct lysp_ctx *ctx, struct hash_table *ht,
+        const char *name, const char *statement, const char *err_detail)
 {
     LY_ERR ret;
     uint32_t hash;
 
-    hash = lyht_hash(name, strlen(name));
+    hash = dict_hash(name, strlen(name));
     ret = lyht_insert(ht, &name, hash, NULL);
     if (ret == LY_EEXIST) {
         if (err_detail) {
@@ -387,7 +388,7 @@ lysp_check_dup_ht_insert(struct lysp_ctx *ctx, struct ly_ht *ht, const char *nam
  */
 static LY_ERR
 lysp_check_dup_typedef(struct lysp_ctx *ctx, struct lysp_node *node, const struct lysp_tpdf *tpdf,
-        struct ly_ht *tpdfs_global)
+        struct hash_table *tpdfs_global)
 {
     struct lysp_node *parent;
     uint32_t hash;
@@ -433,7 +434,7 @@ lysp_check_dup_typedef(struct lysp_ctx *ctx, struct lysp_node *node, const struc
 
     /* check collision with the top-level typedefs */
     if (node) {
-        hash = lyht_hash(name, name_len);
+        hash = dict_hash(name, name_len);
         if (!lyht_find(tpdfs_global, &name, hash, NULL)) {
             LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG,
                     "Duplicate identifier \"%s\" of typedef statement - scoped type collide with a top-level type.", name);
@@ -468,7 +469,7 @@ lysp_id_cmp(void *val1, void *val2, ly_bool UNUSED(mod), void *UNUSED(cb_data))
 LY_ERR
 lysp_check_dup_typedefs(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
-    struct ly_ht *ids_global;
+    struct hash_table *ids_global;
     const struct lysp_tpdf *typedefs;
     LY_ARRAY_COUNT_TYPE u, v;
     uint32_t i;
@@ -495,7 +496,7 @@ lysp_check_dup_typedefs(struct lysp_ctx *ctx, struct lysp_module *mod)
     }
 
 cleanup:
-    lyht_free(ids_global, NULL);
+    lyht_free(ids_global);
     return ret;
 }
 
@@ -527,7 +528,7 @@ lysp_grouping_match(const char *name, struct lysp_node *node)
  */
 static LY_ERR
 lysp_check_dup_grouping(struct lysp_ctx *ctx, struct lysp_node *node, const struct lysp_node_grp *grp,
-        struct ly_ht *grps_global)
+        struct hash_table *grps_global)
 {
     struct lysp_node *parent;
     uint32_t hash;
@@ -566,7 +567,7 @@ lysp_check_dup_grouping(struct lysp_ctx *ctx, struct lysp_node *node, const stru
 
     /* check collision with the top-level groupings */
     if (node) {
-        hash = lyht_hash(name, name_len);
+        hash = dict_hash(name, name_len);
         if (!lyht_find(grps_global, &name, hash, NULL)) {
             LOGVAL_PARSER(ctx, LYVE_SYNTAX_YANG,
                     "Duplicate identifier \"%s\" of grouping statement - scoped grouping collide with a top-level grouping.", name);
@@ -583,7 +584,7 @@ lysp_check_dup_grouping(struct lysp_ctx *ctx, struct lysp_node *node, const stru
 LY_ERR
 lysp_check_dup_groupings(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
-    struct ly_ht *ids_global;
+    struct hash_table *ids_global;
     const struct lysp_node_grp *groupings, *grp_iter;
     LY_ARRAY_COUNT_TYPE u;
     uint32_t i;
@@ -609,7 +610,7 @@ lysp_check_dup_groupings(struct lysp_ctx *ctx, struct lysp_module *mod)
     }
 
 cleanup:
-    lyht_free(ids_global, NULL);
+    lyht_free(ids_global);
     return ret;
 }
 
@@ -625,7 +626,7 @@ LY_ERR
 lysp_check_dup_features(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
     LY_ARRAY_COUNT_TYPE u;
-    struct ly_ht *ht;
+    struct hash_table *ht;
     struct lysp_feature *f;
     LY_ERR ret = LY_SUCCESS;
 
@@ -649,7 +650,7 @@ lysp_check_dup_features(struct lysp_ctx *ctx, struct lysp_module *mod)
     }
 
 cleanup:
-    lyht_free(ht, NULL);
+    lyht_free(ht);
     return ret;
 }
 
@@ -657,7 +658,7 @@ LY_ERR
 lysp_check_dup_identities(struct lysp_ctx *ctx, struct lysp_module *mod)
 {
     LY_ARRAY_COUNT_TYPE u;
-    struct ly_ht *ht;
+    struct hash_table *ht;
     struct lysp_ident *i;
     LY_ERR ret = LY_SUCCESS;
 
@@ -681,7 +682,7 @@ lysp_check_dup_identities(struct lysp_ctx *ctx, struct lysp_module *mod)
     }
 
 cleanup:
-    lyht_free(ht, NULL);
+    lyht_free(ht);
     return ret;
 }
 
@@ -920,7 +921,7 @@ lys_get_module_without_revision(struct ly_ctx *ctx, const char *name)
     struct lys_module *mod, *mod_impl;
     uint32_t index;
 
-    /* try to find module with LYS_MOD_IMPORTED_REV flag */
+    /* Try to find module with LYS_MOD_IMPORTED_REV flag. */
     index = 0;
     while ((mod = ly_ctx_get_module_iter(ctx, &index))) {
         if (!strcmp(mod->name, name) && (mod->latest_revision & LYS_MOD_IMPORTED_REV)) {
@@ -928,19 +929,17 @@ lys_get_module_without_revision(struct ly_ctx *ctx, const char *name)
         }
     }
 
-    /* try to find the implemented module */
+    /* Try to find the implemented module. */
     mod_impl = ly_ctx_get_module_implemented(ctx, name);
-    if (mod) {
-        if (mod_impl && (mod != mod_impl)) {
-            LOGVRB("Implemented module \"%s@%s\" is not used for import, revision \"%s\" is imported instead.",
-                    mod_impl->name, mod_impl->revision, mod->revision);
-        }
+    if (mod && mod_impl) {
+        LOGVRB("Implemented module \"%s@%s\" is not used for import, revision \"%s\" is imported instead.",
+                mod_impl->name, mod_impl->revision, mod->revision);
         return mod;
     } else if (mod_impl) {
         return mod_impl;
     }
 
-    /* try to find the latest module in the current context */
+    /* Try to find the latest module in the current context. */
     mod = ly_ctx_get_module_latest(ctx, name);
 
     return mod;
@@ -950,8 +949,10 @@ lys_get_module_without_revision(struct ly_ctx *ctx, const char *name)
  * @brief Check if a circular dependency exists between modules.
  *
  * @param[in] ctx libyang context for log an error.
- * @param[in,out] mod Examined module which is set to NULL if the circular dependency is detected.
- * @return LY_SUCCESS if no circular dependecy is detected, otherwise LY_EVALID.
+ * @param[in,out] mod Examined module which is set to NULL
+ * if the circular dependency is detected.
+ * @return LY_SUCCESS if no circular dependecy is detected,
+ * otherwise LY_EVALID.
  */
 static LY_ERR
 lys_check_circular_dependency(struct ly_ctx *ctx, struct lys_module **mod)
@@ -1183,6 +1184,7 @@ lysp_inject_submodule(struct lysp_ctx *pctx, struct lysp_include *inc)
         DUP_STRING_RET(PARSER_CTX(pctx), inc->name, inc_new->name);
         DUP_STRING_RET(PARSER_CTX(pctx), inc->dsc, inc_new->dsc);
         DUP_STRING_RET(PARSER_CTX(pctx), inc->ref, inc_new->ref);
+        /* TODO duplicate extensions */
         memcpy(inc_new->rev, inc->rev, LY_REV_SIZE);
         inc_new->injected = 1;
     }
