@@ -1,4 +1,4 @@
-/**
+/*
  * @file test_hash_table.c
  * @author: Radek Krejci <rkrejci@cesnet.cz>
  * @brief unit tests for functions from hash_table.c
@@ -19,7 +19,7 @@
 #include "common.h"
 #include "hash_table.h"
 
-struct ly_ht_rec *lyht_get_rec(unsigned char *recs, uint16_t rec_size, uint32_t idx);
+struct ht_rec *lyht_get_rec(unsigned char *recs, uint16_t rec_size, uint32_t idx);
 
 static void
 test_invalid_arguments(void **state)
@@ -83,7 +83,7 @@ static void
 test_ht_basic(void **state)
 {
     uint32_t i;
-    struct ly_ht *ht;
+    struct hash_table *ht;
 
     assert_non_null(ht = lyht_new(8, sizeof(int), ht_equal_clb, NULL, 0));
 
@@ -96,15 +96,15 @@ test_ht_basic(void **state)
     assert_int_equal(LY_ENOTFOUND, lyht_remove(ht, &i, i));
     CHECK_LOG("Invalid argument hash (lyht_remove_with_resize_cb()).", NULL);
 
-    lyht_free(ht, NULL);
+    lyht_free(ht);
 }
 
 static void
 test_ht_resize(void **state)
 {
     uint32_t i;
-    struct ly_ht_rec *rec;
-    struct ly_ht *ht;
+    struct ht_rec *rec;
+    struct hash_table *ht;
 
     assert_non_null(ht = lyht_new(8, sizeof(int), ht_equal_clb, NULL, 1));
     assert_int_equal(8, ht->size);
@@ -153,7 +153,7 @@ test_ht_resize(void **state)
     }
 
     /* cleanup */
-    lyht_free(ht, NULL);
+    lyht_free(ht);
 }
 
 static void
@@ -162,8 +162,8 @@ test_ht_collisions(void **UNUSED(state))
 #define GET_REC_INT(rec) (*((uint32_t *)&(rec)->val))
 
     uint32_t i;
-    struct ly_ht_rec *rec;
-    struct ly_ht *ht;
+    struct ht_rec *rec;
+    struct hash_table *ht;
 
     assert_non_null(ht = lyht_new(8, sizeof(int), ht_equal_clb, NULL, 1));
 
@@ -253,7 +253,7 @@ test_ht_collisions(void **UNUSED(state))
         assert_int_equal(rec->hits, 0);
     }
 
-    lyht_free(ht, NULL);
+    lyht_free(ht);
 }
 
 int
