@@ -55,7 +55,6 @@ const char *schema_a = "module a {\n"
         "  anydata any {\n"
         "    config false;\n"
         "  }\n"
-        "  anyxml anyx;\n"
         "  leaf-list ll2 {\n"
         "    config false;\n"
         "    type string;\n"
@@ -291,7 +290,6 @@ test_path(void **state)
 
     ret = lyd_new_path2(root, NULL, "/a:c2/l3[1]", NULL, 0, 0, 0, NULL, &node);
     assert_int_equal(ret, LY_EEXIST);
-    CHECK_LOG_CTX("Path \"/a:c2/l3[1]\" already exists.", "Data location \"/a:c2/l3[1]\".");
 
     ret = lyd_new_path2(root, NULL, "/a:c2/l3[2]/x", "val2", 0, 0, 0, NULL, &node);
     assert_int_equal(ret, LY_SUCCESS);
@@ -350,7 +348,6 @@ test_path(void **state)
 
     ret = lyd_new_path2(root, NULL, "/a:ll2[1]", "", 0, 0, 0, NULL, &node);
     assert_int_equal(ret, LY_EEXIST);
-    CHECK_LOG_CTX("Path \"/a:ll2[1]\" already exists.", "Data location \"/a:ll2[1]\".");
 
     ret = lyd_new_path2(root, NULL, "/a:ll2[2]", "val2", 0, 0, 0, NULL, &node);
     assert_int_equal(ret, LY_SUCCESS);
@@ -365,7 +362,6 @@ test_path(void **state)
 
     ret = lyd_new_path2(root, NULL, "/a:ll2[3][.='val3']", NULL, 0, 0, 0, NULL, &node);
     assert_int_equal(ret, LY_EVALID);
-    CHECK_LOG_CTX("Unparsed characters \"[.='val3']\" left at the end of path.", NULL);
 
     lyd_print_mem(&str, root, LYD_XML, LYD_PRINT_WITHSIBLINGS);
     assert_string_equal(str,
@@ -395,55 +391,6 @@ test_path(void **state)
             "}\n");
     free(str);
     lyd_free_siblings(root);
-
-    /* anyxml */
-    ret = lyd_new_path2(NULL, UTEST_LYCTX, "/a:anyx", "<a/><b/><c/>", 0, LYD_ANYDATA_XML, 0, &root, NULL);
-    assert_int_equal(ret, LY_SUCCESS);
-    assert_non_null(root);
-
-    lyd_print_mem(&str, root, LYD_XML, LYD_PRINT_WITHSIBLINGS);
-    assert_string_equal(str,
-            "<anyx xmlns=\"urn:tests:a\">\n"
-            "  <a/>\n"
-            "  <b/>\n"
-            "  <c/>\n"
-            "</anyx>\n");
-    free(str);
-    lyd_print_mem(&str, root, LYD_JSON, LYD_PRINT_WITHSIBLINGS);
-    assert_string_equal(str,
-            "{\n"
-            "  \"a:anyx\": {\n"
-            "    \"a\": [null],\n"
-            "    \"b\": [null],\n"
-            "    \"c\": [null]\n"
-            "  }\n"
-            "}\n");
-    free(str);
-    lyd_free_siblings(root);
-
-    ret = lyd_new_path2(NULL, UTEST_LYCTX, "/a:anyx", "{\"a\":[null],\"b\":[null],\"c\":[null]}", 0, LYD_ANYDATA_JSON, 0, &root, NULL);
-    assert_int_equal(ret, LY_SUCCESS);
-    assert_non_null(root);
-
-    lyd_print_mem(&str, root, LYD_XML, LYD_PRINT_WITHSIBLINGS);
-    assert_string_equal(str,
-            "<anyx xmlns=\"urn:tests:a\">\n"
-            "  <a/>\n"
-            "  <b/>\n"
-            "  <c/>\n"
-            "</anyx>\n");
-    free(str);
-    lyd_print_mem(&str, root, LYD_JSON, LYD_PRINT_WITHSIBLINGS);
-    assert_string_equal(str,
-            "{\n"
-            "  \"a:anyx\": {\n"
-            "    \"a\": [null],\n"
-            "    \"b\": [null],\n"
-            "    \"c\": [null]\n"
-            "  }\n"
-            "}\n");
-    free(str);
-    lyd_free_siblings(root);
 }
 
 static void
